# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["SessionCreateParams", "SessionContext"]


class SessionCreateParams(TypedDict, total=False):
    concurrency: int
    """Number of sessions to create concurrently (check your plan limit)"""

    is_selenium: Annotated[bool, PropertyInfo(alias="isSelenium")]
    """Enable Selenium mode for the browser session (default is false).

    Use this when you plan to connect to the browser session via Selenium.
    """

    proxy_url: Annotated[str, PropertyInfo(alias="proxyUrl")]
    """Custom proxy URL for the browser session.

    Overrides useProxy, disabling Steel-provided proxies in favor of your specified
    proxy. Format: http(s)://username:password@hostname:port
    """

    session_context: Annotated[SessionContext, PropertyInfo(alias="sessionContext")]
    """Session context data to be used in the created session.

    Sessions will start with an empty context by default.
    """

    session_id: Annotated[str, PropertyInfo(alias="sessionId")]
    """Optional custom UUID for the session"""

    solve_captcha: Annotated[bool, PropertyInfo(alias="solveCaptcha")]
    """Enable automatic captcha solving. Default is false."""

    api_timeout: Annotated[int, PropertyInfo(alias="timeout")]
    """Session timeout duration in milliseconds. Default is 900000 (15 minutes)."""

    use_proxy: Annotated[bool, PropertyInfo(alias="useProxy")]
    """Enable Steel-provided residential proxy usage for the browser session.

    Default is false, which routes requests through datacenter proxies.
    """

    user_agent: Annotated[str, PropertyInfo(alias="userAgent")]
    """Custom user agent string for the browser session"""


class SessionContext(TypedDict, total=False):
    cookies: Iterable[Dict[str, object]]
    """Cookies to initialize in the session"""

    local_storage: Annotated[Iterable[Dict[str, object]], PropertyInfo(alias="localStorage")]
    """Local storage items to initialize in the session"""
